function [NumGen, Edfmax, PTotal] = TransitionMonte(ProblemFunction, DisplayFlag, PopSize, RandSeed, nMonte)

% Compute the theoretical quantities related to the Simple BBO algorithm.
% INPUTS: ProblemFunction = address of cost function
%         DisplayFlag = whether or not to display results during program execution
%         PopSize = array of population sizes to use
%         RandSeed = random number seed
%         nMonte = number of Monte Carlo runs for each population size

global fbar fmin fmax sigma dimension

dimension = 4; % dimension of problem
if ~exist('ProblemFunction', 'var')
    ProblemFunction = @Schwefel226;
end
if ~exist('DisplayFlag', 'var')
    DisplayFlag = false;
end
if ~exist('PopSize', 'var')
    PopSize = [4 16 64]; % vector of population sizes
end
if ~exist('RandSeed', 'var')
    RandSeed = round(sum(100*clock));
end
if ~exist('nMonte', 'var')
    nMonte = 10000; % number of Monte Carlo simulations for each population size
end
rand('state', RandSeed); % initialize random number generator
disp([num2str(nMonte), ' Monte Carlo runs']);

fbarMonte = zeros(1, nMonte);
fminMonte = zeros(1, nMonte);
fmaxMonte = zeros(1, nMonte);
sigmaMonte = zeros(1, nMonte);

for i = 1 : length(PopSize)
    disp(['Population # ', num2str(i), '/', num2str(length(PopSize)), ' - PopSize = ', num2str(PopSize(i))]);
    for run = 1 : nMonte
        [cost] = GetPopStats(ProblemFunction, DisplayFlag, PopSize(i), dimension);
        fminMonte(run) = min(cost);
        fmaxMonte(run) = max(cost);
        fbarMonte(run) = mean(cost);
        sigmaMonte(run) = std(cost);
        if DisplayFlag
            close all
            hist(cost, 50);
            h = findobj(gca,'Type','patch');
            set(h, 'FaceColor',[.8 .8 .8], 'EdgeColor','w');
            xlabel('Cost'); ylabel('Histogram');
            S = functions(ProblemFunction); title([S.function, ', dimension = ', num2str(dimension)]);
            disp(['Mean and std dev of cost = ', num2str(fbarMonte(run)), ', ', num2str(sigmaMonte(run))]);
        end
    end
    fmin = mean(fminMonte);
    fmax = mean(fmaxMonte);
    fbar = mean(fbarMonte);
    sigma = mean(sigmaMonte);
    disp(['fbar, fmin, fmax, sigma = ', num2str(fbar), ', ', num2str(fmin), ', ', num2str(fmax), ', ', num2str(sigma)]);
    [Ps, NumGen, Edf] = Transition(DisplayFlag, PopSize(i));
    disp(['mean # generations to first improvement = ', num2str(NumGen)]);
    disp(['mean improvement = ', num2str(Edf)]);
    disp(['mean Ps = ', num2str(Ps)]);
end
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [cost] = GetPopStats(ProblemFunction, DisplayFlag, PopSize, dimension)

OPTIONS.popsize = PopSize;
OPTIONS.numVar = dimension;
[OPTIONS, MinCost, AvgCost, InitFunction, CostFunction, FeasibleFunction, ...
    MaxParValue, MinParValue, Population] = Init(DisplayFlag, ProblemFunction, [], OPTIONS);
for i = 1 : length(Population)
    cost(i) = Population(i).cost;
end
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Ps, NumGen, Edf] = Transition(DisplayFlag, PopSize)

global fbar fmin fmax sigma dimension EPS
global mu1 sigma1
global deltay

EPS = 1e-8; % integral convergence tolerance

% Compute Ps assuming that each SIV contributes a uniformly distributed proportion of the fitness,
% but that the fitness is Gaussian.
s = dimension;
sigma1 = sigma * sqrt(3 / s);
ymax = fmin / s + sigma1;
ymin = fmin / s - sigma1;
ymin = max(ymin, 0);
deltay = ymax - ymin;
zmax = fmax / s + sigma1;
zmin = fmin / s - sigma1;
zmin = max(zmin, 0);
try
    Ps = triplequad(@PsUniformInt, fmin, fmax, ymin, zmax, zmin, zmax, EPS);
catch
end
if DisplayFlag
    disp(['Ps (Gaussian/uniform) = ', num2str(Ps)]);
end
NumGen = log(0.5) / log(1-Ps);
if DisplayFlag
    disp(['Expected # generations before first improvement = ', num2str(NumGen)]);
end

% Compute the expected value of the new best fitness given that migration results
% in an improvement of the best fitness in the population
sigma1 = sigma * sqrt(2 / dimension);
try
    Efmin = dblquad(@EfminAltInt, 0, fmax, fmin, fmax, EPS);
catch
end
fbar = (fmax + fmin) / 2;
Edf = fmin - (fbar - Efmin);
if DisplayFlag
    disp(['Expected improvement given migration (fbar = mid(fmin,fmax)) = ', num2str(Edf)]);
end

% Create transition matrix
N = PopSize - 1 - PopSize * Ps;
T = N * eye(PopSize) + Ps * ones(PopSize, PopSize);
T = T / (PopSize - 1);
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Integrand] = EfminAltInt(x, y)
% x = vector, y = scalar
global fbar fmin fmax sigma sigma1 dimension
mu1 = (y - fmin) / dimension;
Normx = 0.5 * (erf((fmax - mu1) / sigma1 / sqrt(2)) - erf((y - fmin - mu1) / sigma1 / sqrt(2)));
Normy = 0.5 * (erf((fmax - fbar) / sigma / sqrt(2)) - erf((fmin - fbar) / sigma / sqrt(2)));
if Normx <= eps
    Integrand = x / (fmax - (y - fmin));
else   
    Integrand = x .* exp(-(x - mu1).^2 / 2 / sigma1^2) / sigma1 / sqrt(2*pi) / Normx;
end
Integrand = Integrand .* exp(-(y - fbar).^2 / 2 / sigma^2) / sigma / sqrt(2*pi) / Normy;
Integrand(x < y - fmin) = 0;
return 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Integrand] = PsUniformInt(f, y, z)
% Integrand for computing Ps given that each SIV contributes uniformly distributed fitness proportion,
% and given that the fitness itself is normally distributed.
% f = vector, y = scalar, z = scalar
global deltay sigma sigma1 fbar fmin dimension
zmin = f / dimension - sigma1;
zmin = max(zmin, 0);
zmax = f / dimension + sigma1;
deltaz = zmax - zmin;
Integrand = 1 / deltay ./ deltaz;
Integrand = Integrand .* (1 / sigma / sqrt(2*pi)) .* exp(-(f - fbar).^2 / 2 / sigma^2);
deltaf = f - fmin;
Integrand(y > z - deltaf) = 0;
Integrand(z < zmin) = 0;
Integrand(z > zmax) = 0;
return